angular.module('news.service.news', ['categoryService'])
  .service('newsService', function($categoryService) {

    this.buildNews = function(data) {
      var news = new News(data.id);
      updateNewsBase(data, news);
      return news;
    };

    /**
     * 更新 新闻基本信息
     * @param src
     * @param dest
     */
    this.updateNewsBase = function(src, dest) {
      if (src === undefined || dest === undefined) return;
      dest.categories = src.categories;
      dest.images = src.images;
      dest.publish_time = src.publish_time;
      dest.summary = getConvertedNewsStr(src.summary);
      dest.tags = src.tags;
      dest.title = getConvertedNewsStr(src.title);
      dest.topic_flag = src.topic_flag;
      dest.topics = src.topics;
      dest.type = src.type;
    };

    this.getConvertedNewsStr = function(str) {
      if (str === undefined || str.length <= 0) return "";
      return replaceAngleBracket(str);
    };

    /**
     * 更新 新闻详情信息
     * @param src
     * @param dest
     */
    this.updateNewsDetail = function(src, dest) {
      if (src === undefined || dest === undefined) return;
      dest.content = parse_news_image(src.content);
      dest.create_time = src.create_time;
      dest.creator_id = src.creator_id;
      dest.creator_name = src.creator_name;
      dest.source = src.source;
      dest.update_time = src.update_time;
    };

    /**
     * 更新 新闻的互动信息
     * @param src
     * @param dest
     */
    this.updateNewsInteractions = function(src, dest) {
      if (dest.praise !== undefined) {
        src.praiseNum = dest.praise; // 点赞数量
      }
      if (dest.comment !== undefined) {
        src.commentsNum = dest.comment; // 评论数量
      }
      if (dest.is_praised !== undefined) {
        src.isPraise = dest.is_praised; // 是否已经点赞
      }
    };

    /**
     * 插入或更新 新闻
     * @param news
     * @returns {*}
     */
    this.insertOrUpdateCategoryNews = function(category, news) {

      if (news.id === undefined) return false;

      var found = findCategoryNews(category, news.id);
      var needToInsert = found === undefined;
      if (needToInsert) {
        category.newsArray.push(buildNews(news));
      } else {
        updateNewsBase(news, found);
      }

      return needToInsert;
    };

    /**
     * 获取 指定分类中的 新闻
     * @param categoryID
     * @returns {*}
     */
    this.findCategoryNews = function(category, newID) {
      return findItem(category.newsArray, newID);
    };

    /**
     * 根据新闻ID 删除新闻
     * @param newsID
     */
    this.deleteNews = function(category, news) {
      for (var i = 0; i < category.newsArray.length; i++) {
        if (category.newsArray[i].id === news.id) {
          category.newsArray.splice(i, 1);
        }
      }
    };

    /**
     * 判断 新闻是否属于 分类
     * @param news
     * @param categoryID
     * @returns {boolean}
     */
    this.isNewsBelongCategory = function(news, categoryID) {
      if (news === undefined) return false;

      var categories = news.categories;
      for (var i = 0; i < categories.length; i++) {
        if (categories[i] === categoryID) {
          return true;
        }
      }
      return false;
    };

    /**
     * 判断 是否是图文新闻
     * @param news
     * @returns {boolean}
     */
    this.isTypeOfNews = function(news) {
      if (news === undefined) return false;
      var type = news.type;
      return type === 0 || type === 1;
    };

    /**
     * 判断 是否是图集新闻
     * @param news
     * @returns {boolean}
     */
    this.isTypeOfGallery = function(news) {
      if (news === undefined) return false;
      var type = news.type;
      return type === 2;
    };

    /**
     * 判断 是否是专题
     * @param news
     * @returns {boolean}
     */
    this.isTypeOfTopic = function(news) {
      if (news === undefined) return false;
      return (news.topics !== undefined) && (news.topics.length >= 1);
    };

    /**
     * 判断 是否是专题 入口
     * @param news
     * @returns {boolean}
     */
    this.isTypeOfTopicEntrance = function(news) {
      if (news === undefined) return false;
      return news.topic_flag === 0;
    };

    /**
     * 获取新闻 热度
     * @param news
     * @returns {*}
     */
    this.getNewsHot = function(news) {
      if (news === undefined) return INVALID;
      var praiseNum = news.praiseNum;
      var commentsNum = news.commentsNum;

      if (praiseNum === INVALID && commentsNum === INVALID) return 0;

      if (praiseNum === INVALID) praiseNum = 0;
      if (commentsNum === INVALID) commentsNum = 0;

      return praiseNum + commentsNum;
    };

    /**
     * 获取 属于分类的新闻 数组对象
     * @param categoryID
     * @param allNews
     * @returns {Array}
     */
    this.getCategoryNewsArray = function(categoryID, allNews) {
      var result = [];
      if (categoryID !== undefined || allNews !== undefined) {
        for (var i = 0, max = allNews.length; i < max; i++) {
          var news = allNews[i];
          if (news !== undefined && isNewsBelongCategory(news, categoryID)) {
            // 新闻属于此分类
            if (isTypeOfTopic(news)) {
              // 属于此分类，并且是 专题
              if (isTypeOfTopicEntrance(news)) {
                // 专题入口，加入
                insertOrUpdate(result, news);
              }
            } else {
              // 非专题，加入
              insertOrUpdate(result, news);
            }
          }
        }
      }
      return result;
    };

    /**
     * 获取 指定的分类的新闻 最新新闻条目
     * @param categoryID
     * @param requestSize
     * @param callback
     */
    this.getCategoryNewsLatest = function(category, requestSize, callback) {
      if (category === undefined || category.id === undefined) {
        callback(category, FAILURE);
        return;
      }

      var maxts = 0;
      if (category.maxTs != INVALID) {
        maxts = category.maxTs;
      }

      log("getCategoryNewsLatest BEGIN :: " + category.id + " , size :: " + requestSize + " :: maxts-->" + maxts);

      $.getJSON("/news", {
        "category": category.id,
        "maxts": maxts,
        "size": requestSize
      }, function(data) {
        log("getCategoryNewsLatest :: " + category.id + " :: " + data.length);

        $categoryService.markCategoryNewsRequested(category);

        $categoryService.markCategoryHasOlderNews(category, true);

        if (data === undefined || data.length === 0) {
          callback(category, EMPTY);
          return;
        }

        var appended = 0;
        category.appendSize = appended;

        var requestedNewsIDs = [];
        for (var i = 0, max = data.length; i < max; i++) {
          var news = data[i];
          requestedNewsIDs.push(news.id);
          if (insertOrUpdateCategoryNews(news)) {
            appended++;
          }

          $categoryService.updateCategoryTimemark(category, news.publish_time);
        }

        getNewsInteractionsCounterBatch(requestedNewsIDs, function() {
          onEvent(EVENT_ELEMENT_REFRESH);
        });

        category.appendSize = appended;

        if (data.length > 0) {
          $categoryService.markCategoryHasNews(category);
        }

        mergeTopicNewsEntrance();

        category.newsArray.length = 0;
        category.newsArray = getCategoryNewsArray(category.id, mCategoryNews);

        //storeCategoryNews();

        callback(category, SUCCESS);

      }, function(data, statusCode) {
        log("getCategoryNewsLatest :: error");
        log(data);
        callback(category, FAILURE);
      });

    };

    /**
     * 分页请求 新闻
     * @param categoryID
     * @param requestSize
     * @param callback
     */
    this.getCategoryNewsOlder = function(category, requestSize, callback) {
      if (category === undefined || category.id === undefined) {
        callback(category, FAILURE);
        return;
      }

      log("getCategoryNewsOlder :: BEGIN :: " + category.id);
      $.getJSON("/news", {
        "category": category.id,
        "mints": category.minTs !== undefined ? category.minTs : 0,
        "size": requestSize
      }, function(data) {
        markCategoryNewsRequested(category);

        log("getCategoryNewsOlder");
        log(data);

        if (data === undefined || data.length === 0) {
          markCategoryHasOlderNews(category, false);
          callback(category, EMPTY);
          return;
        }

        var requestedNewsIDs = [];
        for (var i = 0, max = data.length; i < max; i++) {
          var news = data[i];
          requestedNewsIDs.push(news.id);

          insertOrUpdateCategoryNews(news);
          updateCategoryTimemark(category, news.publish_time);
        }

        getNewsInteractionsCounterBatch(requestedNewsIDs, function() {});

        mergeTopicNewsEntrance();

        category.newsArray.length = 0;
        category.newsArray = getCategoryNewsArray(category.id, mCategoryNews);

        //storeCategoryNews();

        callback(category, SUCCESS);

      }, function(data, statusCode) {
        log("getCategoryNewsOlder :: error");
        log(data);

        // 无网络，标记无较早的新闻
        markCategoryHasOlderNews(category, false);

        callback(category, FAILURE);
      });
    };


  });
